/****************************************************************************
 * Projekt - Messtechnik 1
 * (c) 2017 softwareentwicklung-als-prozess.de
 * Dokumentation s. Blog
 ***************************************************************************/
// Includes -----------------------------------------------------------------
#include <SPI.h>

// OLED SSD1306 128x64 I2C
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

// Ethernet 
#include <UIPEthernet.h>

// PubSubClient
#include <PubSubClient.h>

// Defines -----------------------------------------------------------------
//OLED Display
#define OLED_RESET 4
#define BLANK  (char)32 // ASCII Leerzeichen
#define EMPTY  (char)48 // ASCII 0
#define ENDE   (char)0  // Stringende
#define PUNKT  (char)46 // ASCII .
#define DPUNKT (char)58 // ASCII :
#define ZWEI   (char)50 // ASCII 2

// Fehlermeldungen
#define ERR_OK     (char)0
#define ERR_NODHCP (char)1
#define ERR_ETH    (char)3

// Variablen -----------------------------------------------------------------
// OLED
Adafruit_SSD1306 display(OLED_RESET);

// MQTT
char tmpstr[3] = {EMPTY,EMPTY,ENDE};
char humstr[3] = {EMPTY,EMPTY,ENDE};
char datum[11] = {EMPTY,EMPTY,PUNKT,EMPTY,EMPTY,PUNKT,ZWEI,EMPTY,EMPTY,EMPTY,ENDE};
char zeit[6]   = {EMPTY,EMPTY,DPUNKT,EMPTY,EMPTY,ENDE};
const char top1[] = "Wohnung/Raum1/Temperatur/Messwert";
const char top2[] = "Wohnung/Raum1/Temperatur/Messzeit";
const char top3[] = "Wohnung/Raum1/Luftfeuchte/Messwert";
const char top4[] = "Wohnung/Raum1/Luftfeuchte/Messzeit";
bool neuerwert = false;

// Ethernet
byte mac[] = { 0x12,0x34,0x56,0x78,0x9A,0xBC };
EthernetClient client;

// PubSubClient
IPAddress    pscsrv(192, 168, 188, 28);
unsigned int pscport = 1883;
PubSubClient pscclient(client);

// Funktionen ----------------------------------------------------------------
// MQTT
void MQTTReconnect() 
{
  // Loop until we're reconnected
  while (!pscclient.connected()) 
  {
    Serial.print("MQTT Verbindungsaufbau... ");
    if (pscclient.connect("JGE-Sub")) 
    {
      Serial.println("MQTT verbunden");
      pscclient.subscribe(top1);
      pscclient.subscribe(top2);
      pscclient.subscribe(top3);
      pscclient.subscribe(top4);
    } 
    else 
    {
      Serial.print(F("MQTT Fehler, Status = "));
      Serial.print(pscclient.state());
      Serial.println(" Naechster Versuch in 5 Sekunden");
      delay(5000);
    }
  }
}

void MQTTCallback(char* topic, byte* payload, unsigned int length) 
{
  int i = 0;
  
  Serial.print("Nachricht angekommen [");
  Serial.print(topic);
  Serial.print("] ");
  for (i = 0; i < length; i++) 
  {
    Serial.print((char)payload[i]);
  }
  Serial.println();
  if (strcmp(topic,top1) == 0)
  {
    tmpstr[0] = payload[0];
    tmpstr[1] = payload[1];
  }
  if (strcmp(topic,top2) == 0)
  {
    datum[0] = payload[0];
    datum[1] = payload[1];
    datum[3] = payload[3];
    datum[4] = payload[4];
    datum[8] = payload[8];
    datum[9] = payload[9];

    zeit[0] = payload[11];
    zeit[1] = payload[12];
    zeit[3] = payload[14];
    zeit[4] = payload[15];
  }
  if (strcmp(topic,top3) == 0)
  {
    humstr[0] = payload[0];
    humstr[1] = payload[1];
  }  
  if (strcmp(topic,top4) == 0)
  {
    datum[0] = payload[0];
    datum[1] = payload[1];
    datum[3] = payload[3];
    datum[4] = payload[4];
    datum[8] = payload[8];
    datum[9] = payload[9];

    zeit[0] = payload[11];
    zeit[1] = payload[12];
    zeit[3] = payload[14];
    zeit[4] = payload[15];
  }
  neuerwert = true;
}

// OLED
void OLED_ShowValues(void)
{
  display.clearDisplay(); // Splashscreen loeschen
  display.setTextSize(2);
  display.setCursor(0,0);
  display.print(F("tmp = "));
  display.print(tmpstr);
  display.println(F("`C"));
  display.print(F("hum = "));
  display.print(humstr);
  display.println(F("%"));
  display.println(datum);
  display.println(zeit);
  display.display();
}

void OLED_ShowFehler(char fno)
{
  display.clearDisplay(); // Splashscreen loeschen
  display.setTextSize(2); // eigener Startscreen
  display.setCursor(0,0);
  display.println(F("Fehler:"));
  switch(fno)
  {
    case ERR_OK :
      display.println(F("Alles Gut!"));
      display.println(F(":-):-):-)"));
      break;
    case ERR_NODHCP :
      display.println(F("kein DHCP!"));
      display.println(F("Netzwerk ?"));
      break;
    case ERR_ETH :
      display.println(F("Ethernet-"));
      display.println(F("Controller"));
      break;
    default:
      display.println(F("unbekannt!"));
      display.println(F("??????????"));
      break;
  }
  display.display();
}

void setup() 
{
  Serial.begin(9600);
  Serial.println(F("[MQTT-Subscriber-OLED]"));

  // OLED
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.clearDisplay();
  display.setTextColor(WHITE);
  display.setTextSize(3); // eigener Startscreen
  display.setCursor(0,0);
  display.println(F("-------"));
  display.println(F(" SEaP:"));
  display.println(F("-------"));
  display.display();
  delay(2000);

  // MQTT
  pscclient.setServer(pscsrv, pscport);
  pscclient.setCallback(MQTTCallback);

  // Ethernet
  Serial.println(F("Starte Ethernet Init"));
  if (Ethernet.begin(mac) == 0)
  {
    Serial.println(F("Ethernet initialisierung fehler"));
    OLED_ShowFehler(ERR_ETH);
    delay(10000);
  }
  Serial.println(Ethernet.localIP());
  OLED_ShowValues(); // Maske mit Initwerten anzeigen
}

void loop () 
{
  // MQTT
  if (!pscclient.connected()) 
  {
    MQTTReconnect();
  }
  pscclient.loop();

  // OLED
  if (true == neuerwert)
  {
    OLED_ShowValues();
    neuerwert = false;
  }
}

